@@ -222,7 +222,7 @@ is_dcc (struct DCC *dcc)
 	return FALSE;
 }
 
-/* this is called from xchat.c:xchat_misc_checks() every 2 seconds. */
+/* this is called from xchat.c:xchat_misc_checks() every 1 second. */
 
 void
 dcc_check_timeouts (void)
@@ -492,7 +492,10 @@ dcc_write_chat (char *nick, char *text)
 				locale = g_locale_from_utf8 (text, len, NULL, &loc_len, NULL);
 		} else
 		{
-			locale = g_convert (text, len, dcc->serv->encoding, "UTF-8", 0, &loc_len, 0);
+			if (dcc->serv->using_irc)	/* using "IRC" encoding (CP1252/UTF-8 hybrid) */
+				locale = g_convert (text, len, "CP1252", "UTF-8", 0, &loc_len, 0);
+			else
+				locale = g_convert (text, len, dcc->serv->encoding, "UTF-8", 0, &loc_len, 0);
 		}
 
 		if (locale)
@@ -532,7 +535,9 @@ dcc_chat_line (struct DCC *dcc, char *line)
 	if (dcc->serv->using_cp1255)
 		len++;	/* include the NUL terminator */
 
-	if (dcc->serv->encoding == NULL)     /* system */
+	if (dcc->serv->using_irc) /* using "IRC" encoding (CP1252/UTF-8 hybrid) */
+		utf = NULL;
+	else if (dcc->serv->encoding == NULL)     /* system */
 		utf = g_locale_to_utf8 (line, len, NULL, &utf_len, NULL);
 	else
 		utf = g_convert (line, len, "UTF-8", dcc->serv->encoding, 0, &utf_len, 0);
@@ -724,7 +729,7 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 				dcc->destfile_fs = g_strdup (buf);
 
 				old = dcc->destfile;
-				dcc->destfile = g_filename_to_utf8 (buf, -1, 0, 0, 0);
+				dcc->destfile = xchat_filename_to_utf8 (buf, -1, 0, 0, 0);
 
 				EMIT_SIGNAL (XP_TE_DCCRENAME, dcc->serv->front_session,
 								 old, dcc->destfile, NULL, NULL, 0);
@@ -1518,30 +1523,13 @@ abortit:
 }
 
 static gboolean
-dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
+dcc_handle_new_ack (struct DCC *dcc)
 {
-	int len;
 	guint32 ack;
 	char buf[16];
-	int sok = dcc->sok;
+	gboolean done = FALSE;
 
-	len = recv (sok, (char *) &ack, 4, MSG_PEEK);
-	if (len < 1)
-	{
-		if (len < 0)
-		{
-			if (would_block ())
-				return TRUE;
-		}
-		EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
-						 file_part (dcc->file), dcc->nick,
-						 errorstring ((len < 0) ? sock_error () : 0), NULL, 0);
-		dcc_close (dcc, STAT_FAILED, FALSE);
-		return TRUE;
-	}
-	if (len < 4)
-		return TRUE;
-	recv (sok, (char *) &ack, 4, 0);
+	memcpy (&ack, dcc->ack_buf, 4);
 	dcc->ack = ntohl (ack);
 
 	/* this could mess up when xfering >32bit files */
@@ -1563,6 +1551,7 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		sprintf (buf, "%d", dcc->cps);
 		EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
 						 file_part (dcc->file), dcc->nick, buf, NULL, 0);
+		done = TRUE;
 	}
 	else if ((!dcc->fastsend) && (dcc->ack >= (dcc->pos & 0xffffffff)))
 	{
@@ -1576,7 +1565,41 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	/* dcc->ack is only used for CPS and PERCENTAGE calcs from now on... */
 #endif
 
-	return TRUE;
+	return done;
+}
+
+static gboolean
+dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
+{
+	int len;
+
+	while (1)
+	{
+		/* try to fill up 4 bytes */
+		len = recv (dcc->sok, dcc->ack_buf, 4 - dcc->ack_pos, 0);
+		if (len < 1)
+		{
+			if (len < 0)
+			{
+				if (would_block ())	/* ok - keep waiting */
+					return TRUE;
+			}
+			EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
+							 file_part (dcc->file), dcc->nick,
+							 errorstring ((len < 0) ? sock_error () : 0), NULL, 0);
+			dcc_close (dcc, STAT_FAILED, FALSE);
+			return TRUE;
+		}
+
+		dcc->ack_pos += len;
+		if (dcc->ack_pos >= 4)
+		{
+			dcc->ack_pos = 0;
+			if (dcc_handle_new_ack (dcc))
+				return TRUE;
+		}
+		/* loop again until would_block() returns true */
+	}
 }
 
 static gboolean
@@ -1787,7 +1810,7 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 		free (file);
 
 		/* for_files() will use opendir, so we need local FS encoding */
-		path_fs = g_filename_from_utf8 (path, -1, 0, 0, 0);
+		path_fs = xchat_filename_from_utf8 (path, -1, 0, 0, 0);
 		if (path_fs)
 		{
 			recursive = TRUE;
@@ -1806,7 +1829,7 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 	dcc->maxcps = maxcps;
 
 	/* get the local filesystem encoding */
-	file_fs = g_filename_from_utf8 (file, -1, 0, 0, 0);
+	file_fs = xchat_filename_from_utf8 (file, -1, 0, 0, 0);
 
 	if (stat (file_fs, &st) != -1)
 	{
@@ -2097,7 +2120,7 @@ dcc_get_with_destfile (struct DCC *dcc, char *file)
 
 	/* get the local filesystem encoding */
 	g_free (dcc->destfile_fs);
-	dcc->destfile_fs = g_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
+	dcc->destfile_fs = xchat_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
 
 	/* since destfile changed, must check resumability again */
 	is_resumable (dcc);
@@ -2349,7 +2372,7 @@ dcc_add_file (session *sess, char *file, DCC_SIZE size, int port, char *nick, gu
 		strcat (dcc->destfile, file);
 
 		/* get the local filesystem encoding */
-		dcc->destfile_fs = g_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
+		dcc->destfile_fs = xchat_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
 
 		dcc->resumable = 0;
 		dcc->pos = 0;
