@@ -107,21 +107,6 @@ static GSList *hook_list = NULL;
 extern const struct prefs vars[];	/* cfgfiles.c */
 
 
-/* 31 bit string hash function */
-
-static guint32
-str_hash (const char *key)
-{
-	const char *p = key;
-	guint32 h = *p;
-
-	if (h)
-		for (p += 1; *p != '\0'; p++)
-			h = (h << 5) - h + *p;
-
-	return h;
-}
-
 /* unload a plugin and remove it from our linked list */
 
 static int
@@ -430,6 +415,11 @@ plugin_auto_load_cb (char *filename)
 {
 	char *pMsg;
 
+#ifndef WIN32	/* black listed */
+	if (!strcmp (file_part (filename), "dbus.so"))
+		return;
+#endif
+
 	pMsg = plugin_load (ps, filename, NULL);
 	if (pMsg)
 	{
@@ -860,13 +850,19 @@ xchat_printf (xchat_plugin *ph, const char *format, ...)
 void
 xchat_command (xchat_plugin *ph, const char *command)
 {
+	char *conv;
+	int len = -1;
+
 	if (!is_session (ph->context))
 	{
 		DEBUG(PrintTextf(0, "%s\txchat_command called without a valid context.\n", ph->name));
 		return;
 	}
 
+	/* scripts/plugins continue to send non-UTF8... *sigh* */
+	conv = text_validate ((char **)&command, &len);
 	handle_command (ph->context, (char *)command, FALSE);
+	g_free (conv);
 }
 
 void
@@ -970,6 +966,27 @@ const char *
 xchat_get_info (xchat_plugin *ph, const char *id)
 {
 	session *sess;
+	guint32 hash;
+
+	if (!strncmp (id, "event_text", 9))
+		return text_find_format_string ((char *)id + 10);
+
+	hash = str_hash (id);
+	/* do the session independant ones first */
+	switch (hash)
+	{
+	case 0x325acab5:	/* libdirfs */
+		return XCHATLIBDIR;
+
+	case 0x14f51cd8: /* version */
+		return PACKAGE_VERSION;
+
+	case 0xdd9b1abd:	/* xchatdir */
+		return get_xdir_utf8 ();
+
+	case 0xe33f6c4a:	/* xchatdirfs */
+		return get_xdir_fs ();
+	}
 
 	sess = ph->context;
 	if (!is_session (sess))
@@ -978,7 +995,7 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 		return NULL;
 	}
 
-	switch (str_hash (id))
+	switch (hash)
 	{
 	case 0x2de2ee: /* away */
 		if (sess->server->is_away)
@@ -1006,9 +1023,6 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x1c0e99c1: /* inputbox */
 		return fe_get_inputbox_contents (sess);
 
-	case 0x325acab5:	/* libdirfs */
-		return XCHATLIBDIR;
-
 	case 0x6de15a2e:	/* network */
 		return server_get_network (sess->server, FALSE);
 
@@ -1028,9 +1042,6 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x696cd2f: /* topic */
 		return sess->topic;
 
-	case 0x14f51cd8: /* version */
-		return VERSION;
-
 	case 0x506d600b: /* native win_ptr */
 		return fe_gui_info_ptr (sess, 0);
 
@@ -1042,12 +1053,6 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 		case 2: return "hidden";
 		}
 		return NULL;
-
-	case 0xdd9b1abd:	/* xchatdir */
-		return get_xdir_utf8 ();
-
-	case 0xe33f6c4a:	/* xchatdirfs */
-		return get_xdir_fs ();
 	}
 
 	return NULL;
@@ -1184,12 +1189,12 @@ xchat_list_fields (xchat_plugin *ph, const char *name)
 	static const char * const dcc_fields[] =
 	{
 		"iaddress32","icps",		"sdestfile","sfile",		"snick",	"iport",
-		"ipos",		"iresume",	"isize",		"istatus", 	"itype",		NULL
+		"ipos", "iresume", "isize", "isizehigh", "istatus", "itype", NULL
 	};
 	static const char * const channels_fields[] =
 	{
-		"schannel",	"schantypes", "pcontext", "iflags", "iid", "imaxmodes",
-		"snetwork", "snickmodes", "snickprefixes", "sserver", "itype", "iusers",
+		"schannel",	"schantypes", "pcontext", "iflags", "iid", "ilag", "imaxmodes",
+		"snetwork", "snickmodes", "snickprefixes", "iqueue", "sserver", "itype", "iusers",
 		NULL
 	};
 	static const char * const ignore_fields[] =
@@ -1198,7 +1203,7 @@ xchat_list_fields (xchat_plugin *ph, const char *name)
 	};
 	static const char * const notify_fields[] =
 	{
-		"iflags", "snick", "toff", "ton", "tseen", NULL
+		"iflags", "snetworks", "snick", "toff", "ton", "tseen", NULL
 	};
 	static const char * const users_fields[] =
 	{
@@ -1266,9 +1271,17 @@ const char *
 xchat_list_str (xchat_plugin *ph, xchat_list *xlist, const char *name)
 {
 	guint32 hash = str_hash (name);
-	gpointer data = xlist->pos->data;
+	gpointer data = ph->context;
+	int type = LIST_CHANNELS;
 
-	switch (xlist->type)
+	/* a NULL xlist is a shortcut to current "channels" context */
+	if (xlist)
+	{
+		data = xlist->pos->data;
+		type = xlist->type;
+	}
+
+	switch (type)
 	{
 	case LIST_CHANNELS:
 		switch (hash)
@@ -1313,6 +1326,8 @@ xchat_list_str (xchat_plugin *ph, xchat_list *xlist, const char *name)
 	case LIST_NOTIFY:
 		switch (hash)
 		{
+		case 0x4e49ec05:	/* networks */
+			return ((struct notify *)data)->networks;
 		case 0x339763: /* nick */
 			return ((struct notify *)data)->name;
 		}
@@ -1338,10 +1353,17 @@ int
 xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 {
 	guint32 hash = str_hash (name);
-	gpointer data = xlist->pos->data;
-	int tmp;
+	gpointer data = ph->context;
+	int tmp, type = LIST_CHANNELS;
 
-	switch (xlist->type)
+	/* a NULL xlist is a shortcut to current "channels" context */
+	if (xlist)
+	{
+		data = xlist->pos->data;
+		type = xlist->type;
+	}
+
+	switch (type)
 	{
 	case LIST_DCC:
 		switch (hash)
@@ -1357,7 +1379,9 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		case 0xc84dc82d: /* resume */
 			return ((struct DCC *)data)->resumable;
 		case 0x35e001: /* size */
-			return ((struct DCC *)data)->size;
+			return ((struct DCC *)data)->size & 0xffffffff;
+		case 0x3284d523: /* sizehigh */
+			return (((struct DCC *)data)->size >> 32) & 0xffffffff;
 		case 0xcacdcff2: /* status */
 			return ((struct DCC *)data)->dccstat;
 		case 0x368f3a: /* type */
@@ -1379,7 +1403,15 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		case 0xd1b:	/* id */
 			return ((struct session *)data)->server->id;
 		case 0x5cfee87:	/* flags */
-			tmp = ((struct session *)data)->server->have_idmsg;   /* bit 5 */
+			tmp = ((struct session *)data)->tray;            /* bit 9 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->beep;               /* 8 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->color_paste;        /* 7 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->hide_join_part;     /* 6 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->server->have_idmsg; /* 5 */
 			tmp <<= 1;
 			tmp |= ((struct session *)data)->server->have_whox;  /* 4 */
 			tmp <<= 1;
@@ -1391,8 +1423,12 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 			tmp <<= 1;
 			tmp |= ((struct session *)data)->server->connected;  /* 0 */
 			return tmp;
+		case 0x1a192: /* lag */
+			return ((struct session *)data)->server->lag;
 		case 0x1916144c: /* maxmodes */
 			return ((struct session *)data)->server->modes_per_line;
+		case 0x66f1911: /* queue */
+			return ((struct session *)data)->server->sendq_len;
 		case 0x368f3a:	/* type */
 			return ((struct session *)data)->type;
 		case 0x6a68e08: /* users */
@@ -1450,10 +1486,11 @@ int
 xchat_emit_print (xchat_plugin *ph, const char *event_name, ...)
 {
 	va_list args;
+	/* currently only 4 because no events use more than 4.
+		This can be easily expanded without breaking the API. */
 	char *argv[4] = {NULL, NULL, NULL, NULL};
 	int i = 0;
 
-	memset (&argv, 0, sizeof (argv));
 	va_start (args, event_name);
 	while (1)
 	{
