@@ -241,7 +241,7 @@ irc_topic (server *serv, char *channel, char *topic)
 }
 
 static void
-irc_list_channels (server *serv, char *arg)
+irc_list_channels (server *serv, char *arg, int min_users)
 {
 	if (arg[0])
 	{
@@ -250,8 +250,7 @@ irc_list_channels (server *serv, char *arg)
 	}
 
 	if (serv->use_listargs)
-								/* 1234567890123456 */
-		tcp_send_len (serv, "LIST >0,<10000\r\n", 16);
+		tcp_sendf (serv, "LIST >%d,<10000\r\n", min_users - 1);
 	else
 		tcp_send_len (serv, "LIST\r\n", 6);
 }
@@ -342,7 +341,8 @@ process_numeric (session * sess, int n,
 		}
 
 		/* use /NICKSERV */
-		if (strcasecmp (word[7], "DALnet") == 0)
+		if (strcasecmp (word[7], "DALnet") == 0 ||
+			 strcasecmp (word[7], "BRASnet") == 0)
 			serv->nickservtype = 1;
 
 		/* use /NS */
@@ -405,6 +405,7 @@ process_numeric (session * sess, int n,
 			}
 
 			serv->skip_next_userhost = FALSE;
+			break;
 		}
 		else goto def;
 
@@ -924,7 +925,7 @@ process_named_msg (session *sess, char *type, char *word[], char *word_eol[])
 							flood_check (nick, ip, serv, sess, 0);
 						if (strncasecmp (text, "DCC ", 4) == 0)
 							/* redo this with handle_quotes TRUE */
-							process_data_init (word[1], word_eol[1], word, word_eol, TRUE);
+							process_data_init (word[1], word_eol[1], word, word_eol, TRUE, FALSE);
 						ctcp_handle (sess, to, nick, text, word, word_eol, id);
 					} else
 					{
@@ -1012,7 +1013,7 @@ irc_inline (server *serv, char *buf, int len)
 	if (buf[0] == ':')
 	{
 		/* split line into words and words_to_end_of_line */
-		process_data_init (pdibuf, buf, word, word_eol, FALSE);
+		process_data_init (pdibuf, buf, word, word_eol, FALSE, FALSE);
 
 		/* find a context for this message */
 		if (is_channel (serv, word[3]))
@@ -1034,7 +1035,7 @@ irc_inline (server *serv, char *buf, int len)
 
 	} else
 	{
-		process_data_init (pdibuf, buf, word, word_eol, FALSE);
+		process_data_init (pdibuf, buf, word, word_eol, FALSE, FALSE);
 		word[0] = type = word[1];
 		if (plugin_emit_server (sess, type, word, word_eol))
 			goto xit;
