@@ -110,8 +110,13 @@ tcp_send_real (server *serv, char *buf, int len)
 		}
 	} else
 	{
-		locale = g_convert_with_fallback (buf, len, serv->encoding, "UTF-8",
-													 "?", 0, &loc_len, 0);
+		if (serv->using_irc)	/* using "IRC" encoding (CP1252/UTF-8 hybrid) */
+			/* if all chars fit inside CP1252, use that. Otherwise this
+			   returns NULL and we send UTF-8. */
+			locale = g_convert (buf, len, "CP1252", "UTF-8", 0, &loc_len, 0);
+		else
+			locale = g_convert_with_fallback (buf, len, serv->encoding, "UTF-8",
+														 "?", 0, &loc_len, 0);
 	}
 
 	if (locale)
@@ -292,8 +297,9 @@ server_inline (server *serv, char *line, int len)
 	char *utf_line_allocated = NULL;
 
 	/* Checks whether we're set to use UTF-8 charset */
-	if ((serv->encoding == NULL && prefs.utf8_locale) ||
-	    (serv->encoding != NULL &&
+	if (serv->using_irc ||				/* 1. using CP1252/UTF-8 Hybrid */
+		(serv->encoding == NULL && prefs.utf8_locale) || /* OR 2. using system default->UTF-8 */
+	    (serv->encoding != NULL &&				/* OR 3. explicitly set to UTF-8 */
 		 (strcasecmp (serv->encoding, "UTF8") == 0 ||
 		  strcasecmp (serv->encoding, "UTF-8") == 0)))
 	{
@@ -806,6 +812,13 @@ auto_reconnect (server *serv, int send_quit, int err)
 #endif
 		serv->reconnect_away = serv->is_away;
 
+	/* is this server in a reconnect delay? remove it! */
+	if (serv->recondelay_tag)
+	{
+		fe_timeout_remove (serv->recondelay_tag);
+		serv->recondelay_tag = 0;
+	}
+
 	serv->recondelay_tag = fe_timeout_add (del, timeout_auto_reconnect, serv);
 	fe_server_event (serv, FE_SE_RECONDELAY, del);
 }
@@ -1087,6 +1100,7 @@ server_disconnect (session * sess, int sendquit, int err)
 	server *serv = sess->server;
 	GSList *list;
 	char tbuf[64];
+	gboolean shutup = FALSE;
 
 	/* send our QUIT reason */
 	if (sendquit && serv->connected)
@@ -1106,6 +1120,8 @@ server_disconnect (session * sess, int sendquit, int err)
 		sprintf (tbuf, "%d", sess->server->childpid);
 		EMIT_SIGNAL (XP_TE_STOPCONNECT, sess, tbuf, NULL, NULL, NULL, 0);
 		return;
+	case 3:
+		shutup = TRUE;	/* won't print "disconnected" in channels */
 	}
 
 	server_flush_queue (serv);
@@ -1116,9 +1132,10 @@ server_disconnect (session * sess, int sendquit, int err)
 		sess = (struct session *) list->data;
 		if (sess->server == serv)
 		{
-			/* print "Disconnected" to each window using this server */
-			EMIT_SIGNAL (XP_TE_DISCON, sess, errorstring (err), NULL, NULL, NULL,
-							 0);
+			if (!shutup || sess->type == SESS_SERVER)
+				/* print "Disconnected" to each window using this server */
+				EMIT_SIGNAL (XP_TE_DISCON, sess, errorstring (err), NULL, NULL, NULL, 0);
+
 			if (!sess->channel[0] || sess->type == SESS_CHANNEL)
 				clear_channel (sess);
 		}
@@ -1392,9 +1409,9 @@ http_read_line (int print_fd, int sok, char *buf, int len)
 static int
 traverse_http (int print_fd, int sok, char *serverAddr, int port)
 {
-	char buf[256];
-	char auth_data[128];
-	char auth_data2[68];
+	char buf[512];
+	char auth_data[256];
+	char auth_data2[252];
 	int n, n2;
 
 	n = snprintf (buf, sizeof (buf), "CONNECT %s:%d HTTP/1.0\r\n",
@@ -1633,7 +1650,9 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	EMIT_SIGNAL (XP_TE_SERVERLOOKUP, sess, hostname, NULL, NULL, NULL, 0);
 
 	safe_strcpy (serv->servername, hostname, sizeof (serv->servername));
-	safe_strcpy (serv->hostname, hostname, sizeof (serv->hostname));
+	/* overlap illegal in strncpy */
+	if (hostname != serv->hostname)
+		safe_strcpy (serv->hostname, hostname, sizeof (serv->hostname));
 
 #ifdef USE_OPENSSL
 	if (serv->use_ssl)
@@ -1642,7 +1661,7 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 
 		snprintf (cert_file, sizeof (cert_file), "%s/%s.pem",
 					 get_xdir_fs (), server_get_network (serv, TRUE));
-		if (SSL_CTX_use_certificate_file (ctx, cert_file, SSL_FILETYPE_PEM) != 1)
+		if (SSL_CTX_use_certificate_file (ctx, cert_file, SSL_FILETYPE_PEM) == 1)
 			SSL_CTX_use_PrivateKey_file (ctx, cert_file, SSL_FILETYPE_PEM);
 	}
 #endif
@@ -1730,6 +1749,7 @@ server_set_encoding (server *serv, char *new_encoding)
 		/* can be left as NULL to indicate system encoding */
 		serv->encoding = NULL;
 		serv->using_cp1255 = FALSE;
+		serv->using_irc = FALSE;
 	}
 
 	if (new_encoding)
@@ -1741,10 +1761,12 @@ server_set_encoding (server *serv, char *new_encoding)
 		if (space)
 			space[0] = 0;
 
-		/* server_inline() uses this flag */
+		/* server_inline() uses these flags */
 		if (!strcasecmp (serv->encoding, "CP1255") ||
 			 !strcasecmp (serv->encoding, "WINDOWS-1255"))
 			serv->using_cp1255 = TRUE;
+		else if (!strcasecmp (serv->encoding, "IRC"))
+			serv->using_irc = TRUE;
 	}
 }
 
@@ -1796,7 +1818,7 @@ server_set_defaults (server *serv)
 	serv->nick_modes = strdup ("ohv");
 
 	serv->nickcount = 1;
-	serv->nickservtype = 0;
+	serv->nickservtype = 1;
 	serv->end_of_motd = FALSE;
 	serv->is_away = FALSE;
 	serv->supports_watch = FALSE;
