@@ -273,7 +273,7 @@ path_part (char *file, char *path, int pathlen)
 }
 
 char *				/* like strstr(), but nocase */
-nocasestrstr (char *s, char *wanted)
+nocasestrstr (const char *s, const char *wanted)
 {
 	register const int len = strlen (wanted);
 
@@ -462,7 +462,7 @@ strip_color2 (char *src, int len, char *dst, int flags)
 				if (!(flags & STRIP_COLOR)) goto pass_char;
 				rcol = 2;
 				break;
-			case '\001':	/* CL: invisible text (for event formats only) */	/* this takes care of the topic */
+			case HIDDEN_CHAR:	/* CL: invisible text (for event formats only) */	/* this takes care of the topic */
 				if (!(flags & STRIP_HIDDEN)) goto pass_char;
 				break;
 			case '\007':			  /*ATTR_BEEP: */
@@ -491,7 +491,7 @@ strip_hidden_attribute (char *src, char *dst)
 	int len = 0;
 	while (*src != '\000')
 	{
-		if (*src != '\001')
+		if (*src != HIDDEN_CHAR)
 		{
 			*dst++ = *src;
 			len++;
@@ -528,7 +528,7 @@ get_cpu_info (double *mhz, int *cpus)
 			break;
 		if (!strncmp (buf, "cycle frequency [Hz]\t:", 22))	/* alpha */
 		{
-			*mhz = atoi (buf + 23) / 1048576;
+			*mhz = atoi (buf + 23) / 1000000;
 		} else if (!strncmp (buf, "cpu MHz\t\t:", 10))	/* i386 */
 		{
 			*mhz = atof (buf + 11) + 0.5;
@@ -766,6 +766,8 @@ break_while:
       default:
 	if (rfc_tolower(*s) != rfc_tolower(ch))
 	{
+	  if (!*s)
+	    return 0;
 	  m = bm;
 	  s = bs;
 	  goto got_star;
@@ -834,6 +836,7 @@ country_compare (const void *a, const void *b)
 
 static const domain_t domain[] =
 {
+		{"AC", N_("Ascension Island") },
 		{"AD", N_("Andorra") },
 		{"AE", N_("United Arab Emirates") },
 		{"AF", N_("Afghanistan") },
@@ -851,6 +854,7 @@ static const domain_t domain[] =
 		{"ATO", N_("Nato Fiel") },
 		{"AU", N_("Australia") },
 		{"AW", N_("Aruba") },
+		{"AX", N_("Aland Islands") },
 		{"AZ", N_("Azerbaijan") },
 		{"BA", N_("Bosnia and Herzegovina") },
 		{"BB", N_("Barbados") },
@@ -906,13 +910,13 @@ static const domain_t domain[] =
 		{"ER", N_("Eritrea") },
 		{"ES", N_("Spain") },
 		{"ET", N_("Ethiopia") },
+		{"EU", N_("European Union") },
 		{"FI", N_("Finland") },
 		{"FJ", N_("Fiji") },
 		{"FK", N_("Falkland Islands") },
 		{"FM", N_("Micronesia") },
 		{"FO", N_("Faroe Islands") },
 		{"FR", N_("France") },
-		{"FX", N_("France, Metropolitan") },
 		{"GA", N_("Gabon") },
 		{"GB", N_("Great Britain") },
 		{"GD", N_("Grenada") },
@@ -942,6 +946,7 @@ static const domain_t domain[] =
 		{"ID", N_("Indonesia") },
 		{"IE", N_("Ireland") },
 		{"IL", N_("Israel") },
+		{"IM", N_("Isle of Man") },
 		{"IN", N_("India") },
 		{"INFO", N_("Informational") },
 		{"INT", N_("International") },
@@ -950,6 +955,7 @@ static const domain_t domain[] =
 		{"IR", N_("Iran") },
 		{"IS", N_("Iceland") },
 		{"IT", N_("Italy") },
+		{"JE", N_("Jersey") },
 		{"JM", N_("Jamaica") },
 		{"JO", N_("Jordan") },
 		{"JP", N_("Japan") },
@@ -1009,7 +1015,6 @@ static const domain_t domain[] =
 		{"NO", N_("Norway") },
 		{"NP", N_("Nepal") },
 		{"NR", N_("Nauru") },
-		{"NT", N_("Neutral Zone") },
 		{"NU", N_("Niue") },
 		{"NZ", N_("New Zealand") },
 		{"OM", N_("Oman") },
@@ -1061,6 +1066,7 @@ static const domain_t domain[] =
 		{"TH", N_("Thailand") },
 		{"TJ", N_("Tajikistan") },
 		{"TK", N_("Tokelau") },
+		{"TL", N_("East Timor") },
 		{"TM", N_("Turkmenistan") },
 		{"TN", N_("Tunisia") },
 		{"TO", N_("Tonga") },
@@ -1172,7 +1178,8 @@ int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPtr) {
 		if (*src != quote) *buf++ = '\\';
 	    }
 	    *buf++ = *src;
-	} else if (isspace((unsigned char) *src)) {
+	/*} else if (isspace((unsigned char) *src)) {*/
+	} else if (*src == ' ') {
 	    if (*argv[argc]) {
 		buf++, argc++;
 		if (argc == argvAlloced) {
@@ -1227,7 +1234,7 @@ int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPtr) {
 }
 
 int
-util_exec (char *cmd)
+util_exec (const char *cmd)
 {
 	int pid;
 	char **argv;
@@ -1259,6 +1266,31 @@ util_exec (char *cmd)
 #endif
 }
 
+int
+util_execv (char * const argv[])
+{
+	int pid, fd;
+
+#ifndef WIN32
+	pid = fork ();
+	if (pid == -1)
+		return -1;
+	if (pid == 0)
+	{
+		/* Now close all open file descriptors except stdin, stdout and stderr */
+		for (fd = 3; fd < 1024; fd++) close(fd);
+		execv (argv[0], argv);
+		_exit (0);
+	} else
+	{
+		return pid;
+	}
+#else
+	spawnv (_P_DETACH, argv[0], argv);
+	return 0;
+#endif
+}
+
 unsigned long
 make_ping_time (void)
 {
@@ -1391,170 +1423,180 @@ const unsigned char rfc_tolowertab[] =
 	0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
 };*/
 
-/* Code borrowed from gtk-gnutella */
-/* Takes care of moving a file from a temporary download location to a completed location */
-void
-download_move_to_completed_dir (char *dcc_dir, char *dcc_completed_dir, 
-	char *output_name, int dccpermissions)
+/*static int
+rename_utf8 (char *oldname, char *newname)
 {
-	/* Move a complete file to move_file_path */
-	/* mgl: this used to take just a filename and move it between dirs */
-	/* now it takes the full path of the target download and moves it */
+	int sav, res;
+	char *fso, *fsn;
 
-	char dl_src[4096];
-	char dl_dest[4096];
+	fso = xchat_filename_from_utf8 (oldname, -1, 0, 0, 0);
+	if (!fso)
+		return FALSE;
+	fsn = xchat_filename_from_utf8 (newname, -1, 0, 0, 0);
+	if (!fsn)
+	{
+		g_free (fso);
+		return FALSE;
+	}
+
+	res = rename (fso, fsn);
+	sav = errno;
+	g_free (fso);
+	g_free (fsn);
+	errno = sav;
+	return res;
+}
+
+static int
+unlink_utf8 (char *fname)
+{
+	int res;
+	char *fs;
+
+	fs = xchat_filename_from_utf8 (fname, -1, 0, 0, 0);
+	if (!fs)
+		return FALSE;
+
+	res = unlink (fs);
+	g_free (fs);
+	return res;
+}*/
+
+static gboolean
+file_exists_utf8 (char *fname)
+{
+	int res;
+	char *fs;
+
+	fs = xchat_filename_from_utf8 (fname, -1, 0, 0, 0);
+	if (!fs)
+		return FALSE;
+
+	res = access (fs, F_OK);
+	g_free (fs);
+	if (res == 0)
+		return TRUE;
+	return FALSE;
+}
+
+static gboolean
+copy_file (char *dl_src, char *dl_dest, int permissions)	/* FS encoding */
+{
+	int tmp_src, tmp_dest;
+	gboolean ok = FALSE;
 	char dl_tmp[4096];
-	char *output_fname;
 	int return_tmp, return_tmp2;
 
+	if ((tmp_src = open (dl_src, O_RDONLY | OFLAGS)) == -1)
+	{
+		fprintf (stderr, "Unable to open() file '%s' (%s) !", dl_src,
+				  strerror (errno));
+		return FALSE;
+	}
+
+	if ((tmp_dest =
+		 open (dl_dest, O_WRONLY | O_CREAT | O_TRUNC | OFLAGS, permissions)) < 0)
+	{
+		close (tmp_src);
+		fprintf (stderr, "Unable to create file '%s' (%s) !", dl_src,
+				  strerror (errno));
+		return FALSE;
+	}
+
+	for (;;)
+	{
+		return_tmp = read (tmp_src, dl_tmp, sizeof (dl_tmp));
+
+		if (!return_tmp)
+		{
+			ok = TRUE;
+			break;
+		}
+
+		if (return_tmp < 0)
+		{
+			fprintf (stderr, "download_move_to_completed_dir(): "
+				"error reading while moving file to save directory (%s)",
+				 strerror (errno));
+			break;
+		}
+
+		return_tmp2 = write (tmp_dest, dl_tmp, return_tmp);
+
+		if (return_tmp2 < 0)
+		{
+			fprintf (stderr, "download_move_to_completed_dir(): "
+				"error writing while moving file to save directory (%s)",
+				 strerror (errno));
+			break;
+		}
+
+		if (return_tmp < sizeof (dl_tmp))
+		{
+			ok = TRUE;
+			break;
+		}
+	}
+
+	close (tmp_dest);
+	close (tmp_src);
+	return ok;
+}
+
+/* Takes care of moving a file from a temporary download location to a completed location. Now in UTF-8. */
+void
+move_file_utf8 (char *src_dir, char *dst_dir, char *fname, int dccpermissions)
+{
+	char src[4096];
+	char dst[4096];
+	int res, i;
+	char *src_fs;	/* FileSystem encoding */
+	char *dst_fs;
+
 	/* if dcc_dir and dcc_completed_dir are the same then we are done */
-	if (0 == strcmp (dcc_dir, dcc_completed_dir) ||
-		 0 == dcc_completed_dir[0])
+	if (0 == strcmp (src_dir, dst_dir) ||
+		 0 == dst_dir[0])
 		return;			/* Already in "completed dir" */
 
-	/* mgl: since output_name is a full path, we just copy it */
-	strncpy (dl_src, output_name, sizeof(dl_src));
-	dl_src[sizeof(dl_src)-1] = '\0';
+	snprintf (src, sizeof (src), "%s/%s", src_dir, fname);
+	snprintf (dst, sizeof (dst), "%s/%s", dst_dir, fname);
 
-	/* mgl: output_name being a full path, we need to extract the filename */
-	/* off the end of it before continuing */
-
-	/* no path sep or no file after pathsep?  very suspicious, bail now! */
-	if ((NULL == (output_fname = strrchr(output_name, '/')))
-			|| !*(output_fname + 1))
-		return;
-	/* get the next char after the pathsep */
-	++output_fname;
-	/* throw the filename onto the directory name of the completed directory */
-	/* FIXME: dcc_completed_dir is UTF8, not fs encoding! */
-	snprintf (dl_dest, sizeof (dl_dest), "%s/%s", dcc_completed_dir,
-			   output_fname);
-	/* the rest should continue as before, but use output_fname */
-
-	dl_dest[sizeof(dl_dest)-1] = '\0';
-
-	/*
-	 * If, by extraordinary, there is already a file in the "completed dir"
-	 * with the same name, don't overwrite the existing file.
-	 *
-	 * NB: we assume either there is only one gnutella servent running, or if
-	 * several ones are running, that they are configured to use different
-	 * download and completed dirs.
-	 *
-	 *		--RAM, 03/11/2001
-	 */
-
-	if (access (dl_dest, F_OK) == 0)
+	/* already exists in completed dir? Append a number */
+	if (file_exists_utf8 (dst))
 	{
-		int destlen = strlen (dl_dest);
-		int i;
-		struct stat buf;
-
-		/*
-		 * There must be enough room for us to append the ".xx" extensions.
-		 * That's 3 chars, plus the trailing NUL.
-		 */
-
-		if (destlen >= sizeof (dl_dest) - 4)
+		for (i = 0; ; i++)
 		{
-			fprintf(stderr, "Found '%s' in completed dir, and path already too long",
-				output_fname);
-			return;
-		}
-
-		strncpy (dl_tmp, dl_dest, destlen);
-
-		for (i = 1; i < 100; i++)
-		{
-			char ext[4];
-
-			snprintf (ext, 4, ".%02d", i);
-			dl_tmp[destlen] = '\0';				/* Ignore prior attempt */
-			strncat (dl_tmp+destlen, ext, 3);	/* Append .01, .02, ...*/
-			if (-1 == stat (dl_tmp, &buf))
+			snprintf (dst, sizeof (dst), "%s/%s.%d", dst_dir, fname, i);
+			if (!file_exists_utf8 (dst))
 				break;
 		}
-
-		if (i == 100)
-		{
-			fprintf (stderr, "Found '%s' in completed dir, "
-				"and was unable to find another unique name",
-				output_fname);
-			return;
-		}
-
-		strncat (dl_dest+destlen, dl_tmp+destlen, 3);
 	}
 
-	/* First try and link it to the new locatation */
-
-	return_tmp = rename (dl_src, dl_dest);
-
-	if (return_tmp == -1 && (errno == EXDEV || errno == EPERM))
+	/* convert UTF-8 to filesystem encoding */
+	src_fs = xchat_filename_from_utf8 (src, -1, 0, 0, 0);
+	if (!src_fs)
+		return;
+	dst_fs = xchat_filename_from_utf8 (dst, -1, 0, 0, 0);
+	if (!dst_fs)
 	{
-		/* link failed becase either the two paths aren't on the */
+		g_free (src_fs);
+		return;
+	}
+
+	/* first try a simple rename move */
+	res = rename (src_fs, dst_fs);
+
+	if (res == -1 && (errno == EXDEV || errno == EPERM))
+	{
+		/* link failed because either the two paths aren't on the */
 		/* same filesystem or the filesystem doesn't support hard */
 		/* links, so we have to do a copy. */
-
-		int tmp_src, tmp_dest;
-		gboolean ok = FALSE;
-
-		if ((tmp_src = open (dl_src, O_RDONLY | OFLAGS)) == -1)
-		{
-			fprintf (stderr, "Unable to open() file '%s' (%s) !", dl_src,
-					  strerror (errno));
-			return;
-		}
-
-		if ((tmp_dest =
-			 open (dl_dest, O_WRONLY | O_CREAT | O_TRUNC | OFLAGS, dccpermissions)) < 0)
-		{
-			close (tmp_src);
-			fprintf (stderr, "Unable to create file '%s' (%s) !", dl_src,
-					  strerror (errno));
-			return;
-		}
-
-		for (;;)
-		{
-			return_tmp = read (tmp_src, dl_tmp, sizeof (dl_tmp));
-
-			if (!return_tmp)
-			{
-				ok = TRUE;
-				break;
-			}
-
-			if (return_tmp < 0)
-			{
-				fprintf (stderr, "download_move_to_completed_dir(): "
-					"error reading while moving file to save directory (%s)",
-					 strerror (errno));
-				break;
-			}
-
-			return_tmp2 = write (tmp_dest, dl_tmp, return_tmp);
-
-			if (return_tmp2 < 0)
-			{
-				fprintf (stderr, "download_move_to_completed_dir(): "
-					"error writing while moving file to save directory (%s)",
-					 strerror (errno));
-				break;
-			}
-
-			if (return_tmp < sizeof(dl_tmp))
-			{
-				ok = TRUE;
-				break;
-			}
-		}
-
-		close (tmp_dest);
-		close (tmp_src);
-		if (ok)
-			unlink (dl_src);
+		if (copy_file (src_fs, dst_fs, dccpermissions))
+			unlink (src_fs);
 	}
+
+	g_free (dst_fs);
+	g_free (src_fs);
 }
 
 int
@@ -1562,7 +1604,7 @@ mkdir_utf8 (char *dir)
 {
 	int ret;
 
-	dir = g_filename_from_utf8 (dir, -1, 0, 0, 0);
+	dir = xchat_filename_from_utf8 (dir, -1, 0, 0, 0);
 	if (!dir)
 		return -1;
 
@@ -1612,3 +1654,68 @@ token_foreach (char *str, char sep,
 
 	return TRUE;
 }
+
+/* 31 bit string hash functions */
+
+guint32
+str_hash (const char *key)
+{
+	const char *p = key;
+	guint32 h = *p;
+
+	if (h)
+		for (p += 1; *p != '\0'; p++)
+			h = (h << 5) - h + *p;
+
+	return h;
+}
+
+guint32
+str_ihash (const unsigned char *key)
+{
+	const char *p = key;
+	guint32 h = rfc_tolowertab [(guint)*p];
+
+	if (h)
+		for (p += 1; *p != '\0'; p++)
+			h = (h << 5) - h + rfc_tolowertab [(guint)*p];
+
+	return h;
+}
+
+/* features: 1. "src" must be valid, NULL terminated UTF-8 */
+/*           2. "dest" will be left with valid UTF-8 - no partial chars! */
+
+void
+safe_strcpy (char *dest, const char *src, int bytes_left)
+{
+	int mbl;
+
+	while (1)
+	{
+		mbl = g_utf8_skip[*((unsigned char *)src)];
+
+		if (bytes_left < (mbl + 1)) /* can't fit with NULL? */
+		{
+			*dest = 0;
+			break;
+		}
+
+		if (mbl == 1)	/* one byte char */
+		{
+			*dest = *src;
+			if (*src == 0)
+				break;	/* it all fit */
+			dest++;
+			src++;
+			bytes_left--;
+		}
+		else				/* multibyte char */
+		{
+			memcpy (dest, src, mbl);
+			dest += mbl;
+			src += mbl;
+			bytes_left -= mbl;
+		}
+	}
+}
