@@ -338,6 +338,16 @@ log_handler (const gchar   *log_domain,
 
 #endif
 
+int tray_plugin_init (void *, char **, char **, char **, char *);
+int tray_plugin_deinit (void *);
+
+static int
+fe_idle (gpointer data)
+{
+	plugin_add (sess_list->data, NULL, NULL, tray_plugin_init, tray_plugin_deinit, NULL, FALSE);
+	return 0;
+}
+
 void
 fe_new_window (session *sess, int focus)
 {
@@ -361,6 +371,9 @@ fe_new_window (session *sess, int focus)
 	g_log_set_handler ("Gdk", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
 	g_log_set_handler ("Gtk", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
 #endif
+
+	if (!sess_list->next)
+		g_idle_add (fe_idle, NULL);
 }
 
 void
@@ -753,6 +766,9 @@ fe_ctrl_gui (session *sess, int action, int arg)
 		break;
 	case 7:
 		mg_detach (sess, arg);	/* arg: 0=toggle 1=detach 2=attach */
+		break;
+	case 8:
+		setup_apply_real (TRUE, TRUE);
 	}
 }
 
@@ -870,37 +886,64 @@ fe_set_inputbox_contents (session *sess, char *text)
 	}
 }
 
+#ifndef WIN32
+
+static gboolean
+try_browser (const char *browser, const char *arg, const char *url)
+{
+	const char *argv[4];
+	char *path;
+
+	path = g_find_program_in_path (browser);
+	if (!path)
+		return 0;
+
+	argv[0] = path;
+	argv[1] = url;
+	argv[2] = NULL;
+	if (arg)
+	{
+		argv[1] = arg;
+		argv[2] = url;
+		argv[3] = NULL;
+	}
+	xchat_execv (argv);
+	g_free (path);
+	return 1;
+}
+
+#endif
+
 static void
 fe_open_url_locale (const char *url)
 {
 #ifdef WIN32
 	ShellExecute (0, "open", url, NULL, NULL, SW_SHOWNORMAL);
 #else
-	char tbuf[512], *moz;
-
-	/* Think twice about patching this (yes you Debian!). Using gnome-open makes
-      it user friendly, by letting the user change their browser in Gnome:
-      System -> Preferences -> More Preferences -> Preferred Applications.
-      More info at http://xchat.org/faq/#q221 */
-	moz = g_find_program_in_path ("gnome-open"); /* Gnome 2.4+ has this */
-	if (moz)
-	{
-		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
-		g_free (moz);
-		xchat_exec (tbuf);
+	/* universal desktop URL opener (from xdg-utils). Supports gnome,kde,xfce4. */
+	if (try_browser ("xdg-open", NULL, url))
 		return;
+
+	/* try to detect GNOME */
+	if (g_getenv ("GNOME_DESKTOP_SESSION_ID"))
+	{
+		if (try_browser ("gnome-open", NULL, url)) /* Gnome 2.4+ has this */
+			return;
 	}
 
-	moz = g_find_program_in_path ("firefox");
-	if (moz)
+	/* try to detect KDE */
+	if (g_getenv ("KDE_FULL_SESSION"))
 	{
-		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
-		g_free (moz);
-	} else
-	{
-		snprintf (tbuf, sizeof (tbuf), "mozilla -remote 'openURL(%s)'", url);
+		if (try_browser ("kfmclient", "exec", url))
+			return;
 	}
-	xchat_exec (tbuf);
+
+	/* everything failed, what now? just try firefox */
+	if (try_browser ("firefox", NULL, url))
+		return;
+
+	/* fresh out of ideas... */
+	try_browser ("mozilla", NULL, url);
 #endif
 }
 
@@ -956,7 +999,7 @@ fe_server_event (server *serv, int type, int arg)
 				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_JOIN], 1);
 				/* if number of auto-join channels is zero, open joind */
 				if (arg == 0)
-					joind (0, serv);
+					joind_open (serv);
 				break;
 
 			case FE_SE_DISCONNECT:
@@ -965,7 +1008,7 @@ fe_server_event (server *serv, int type, int arg)
 				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_DISCONNECT], 0);
 				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_JOIN], 0);
 				/* close the join-dialog, if one exists */
-				joind (1, serv);
+				joind_close (serv);
 			}
 		}
 		list = list->next;
