@@ -445,6 +445,23 @@ mg_inputbox_cb (GtkWidget *igad, session_gui *gui)
 	free (cmd);
 }
 
+static gboolean
+has_key (char *modes)
+{
+	if (!modes)
+		return FALSE;
+	/* this is a crude check, but "-k" can't exist, so it works. */
+	while (*modes)
+	{
+		if (*modes == 'k')
+			return TRUE;
+		if (*modes == ' ')
+			return FALSE;
+		modes++;
+	}
+	return FALSE;
+}
+
 void
 fe_set_title (session *sess)
 {
@@ -470,10 +487,17 @@ fe_set_title (session *sess)
 					 sess->server->nick, server_get_network (sess->server, TRUE));
 		break;
 	case SESS_CHANNEL:
-		snprintf (tbuf, sizeof (tbuf),
-					 "XChat: %s @ %s / %s (%s)",
-					 sess->server->nick, server_get_network (sess->server, TRUE),
-					 sess->channel, sess->current_modes ? sess->current_modes : "");
+		/* don't display keys in the titlebar */
+		if (has_key (sess->current_modes))
+			snprintf (tbuf, sizeof (tbuf),
+						 "XChat: %s @ %s / %s",
+						 sess->server->nick, server_get_network (sess->server, TRUE),
+						 sess->channel);
+		else
+			snprintf (tbuf, sizeof (tbuf),
+						 "XChat: %s @ %s / %s (%s)",
+						 sess->server->nick, server_get_network (sess->server, TRUE),
+						 sess->channel, sess->current_modes ? sess->current_modes : "");
 		break;
 	case SESS_NOTICES:
 	case SESS_SNOTICES:
@@ -1054,18 +1078,6 @@ mg_tab_close (session *sess)
 	}
 }
 
-static void
-mg_tree_cb (GtkWidget *item, session *sess)
-{
-	if (is_session (sess))
-	{
-		if (sess->gui->is_tab)
-			mg_bring_tofront_sess (sess);
-		else
-			gtk_window_present (GTK_WINDOW (sess->gui->window));
-	}
-}
-
 static void
 mg_colorpaste_cb (GtkCheckMenuItem *item, session *sess)
 {
@@ -1075,6 +1087,14 @@ mg_colorpaste_cb (GtkCheckMenuItem *item, session *sess)
 	GTK_XTEXT (sess->gui->xtext)->color_paste = sess->color_paste;
 }
 
+static void
+mg_traymsg_cb (GtkCheckMenuItem *item, session *sess)
+{
+	sess->tray = FALSE;
+	if (item->active)
+		sess->tray = TRUE;
+}
+
 static void
 mg_beepmsg_cb (GtkCheckMenuItem *item, session *sess)
 {
@@ -1091,53 +1111,6 @@ mg_hidejp_cb (GtkCheckMenuItem *item, session *sess)
 		sess->hide_join_part = FALSE;
 }
 
-static void
-mg_create_sess_tree (GtkWidget *menu)
-{
-	GtkWidget *top_item, *item, *submenu;
-	GSList *list, *ilist;
-	server *serv;
-	session *sess;
-
-	list = serv_list;
-	while (list)
-	{
-		serv = list->data;
-
-		if (serv->servername[0] == 0)
-			top_item = gtk_menu_item_new_with_label (_("<none>"));
-		else
-			top_item = gtk_menu_item_new_with_label (server_get_network (serv, TRUE));
-
-		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), top_item);
-		gtk_widget_show (top_item);
-
-		submenu = gtk_menu_new ();
-
-		ilist = sess_list;
-		while (ilist)
-		{
-			sess = ilist->data;
-			if (sess->server == serv)
-			{
-				if (sess->channel[0] == 0)
-					item = gtk_menu_item_new_with_label (_("<none>"));
-				else
-					item = gtk_menu_item_new_with_label (sess->channel);
-				g_signal_connect (G_OBJECT (item), "activate",
-										G_CALLBACK (mg_tree_cb), sess);
-				gtk_menu_shell_prepend (GTK_MENU_SHELL (submenu), item);
-				gtk_widget_show (item);
-			}
-			ilist = ilist->next;
-		}
-
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (top_item), submenu);
-
-		list = list->next;
-	}
-}
-
 static void
 mg_menu_destroy (GtkWidget *menu, gpointer userdata)
 {
@@ -1145,7 +1118,7 @@ mg_menu_destroy (GtkWidget *menu, gpointer userdata)
 	g_object_unref (menu);
 }
 
-static void
+void
 mg_create_icon_item (char *label, char *stock, GtkWidget *menu,
 							void *callback, void *userdata)
 {
@@ -1326,12 +1299,12 @@ mg_markup_item (GtkWidget *menu, char *text, int arg)
 	gtk_widget_show (item);
 }
 
-static GtkWidget *
+GtkWidget *
 mg_submenu (GtkWidget *menu, char *text)
 {
 	GtkWidget *submenu, *item;
 
-	item = gtk_menu_item_new_with_label (text);
+	item = gtk_menu_item_new_with_mnemonic (text);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
 
@@ -1381,8 +1354,9 @@ mg_create_color_menu (GtkWidget *menu, session *sess)
 static gboolean
 mg_tab_contextmenu_cb (chanview *cv, chan *ch, int tag, gpointer ud, GdkEventButton *event)
 {
-	GtkWidget *menu, *submenu, *item;
+	GtkWidget *menu, *item;
 	session *sess = ud;
+	char buf[256];
 
 	/* shift-click to close a tab */
 	if ((event->state & GDK_SHIFT_MASK) && event->type == GDK_BUTTON_PRESS)
@@ -1398,31 +1372,32 @@ mg_tab_contextmenu_cb (chanview *cv, chan *ch, int tag, gpointer ud, GdkEventBut
 
 	if (tag == TAG_IRC)
 	{
-		item = gtk_menu_item_new_with_label (sess->channel[0] ? sess->channel : _("<none>"));
+		char *name = g_markup_escape_text (sess->channel[0] ? sess->channel : _("<none>"), -1);
+		snprintf (buf, sizeof (buf), "<span foreground=\"#3344cc\"><b>%s</b></span>", name);
+		g_free (name);
+		item = gtk_menu_item_new_with_label ("");
+		gtk_label_set_markup (GTK_LABEL (GTK_BIN (item)->child), buf);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		gtk_widget_show (item);
 
-		submenu = gtk_menu_new ();
+		/* separator */
+		item = gtk_menu_item_new ();
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		gtk_widget_show (item);
 
-		menu_toggle_item (_("Beep on message"), submenu, mg_beepmsg_cb, sess,
+		menu_toggle_item (_("Beep on message"), menu, mg_beepmsg_cb, sess,
 								sess->beep);
+		if (prefs.gui_tray)
+			menu_toggle_item (_("Blink tray on message"), menu, mg_traymsg_cb, sess,
+									sess->tray);
 		if (sess->type == SESS_CHANNEL)
-			menu_toggle_item (_("Show join/part messages"), submenu, mg_hidejp_cb,
+			menu_toggle_item (_("Show join/part messages"), menu, mg_hidejp_cb,
 									sess, !sess->hide_join_part);
-		menu_toggle_item (_("Color paste"), submenu, mg_colorpaste_cb, sess,
+		menu_toggle_item (_("Color paste"), menu, mg_colorpaste_cb, sess,
 								sess->color_paste);
 
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
-		gtk_widget_show (item);
 	}
 
-	item = gtk_menu_item_new_with_label (_("Go to"));
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-
-	submenu = gtk_menu_new ();
-	mg_create_sess_tree (submenu);
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
-	gtk_widget_show (item);
-
 	/* separator */
 	item = gtk_menu_item_new ();
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -1430,17 +1405,18 @@ mg_tab_contextmenu_cb (chanview *cv, chan *ch, int tag, gpointer ud, GdkEventBut
 
 	mg_create_icon_item (_("_Close Tab"), GTK_STOCK_CLOSE, menu,
 								mg_destroy_tab_cb, ch);
-	/*if (tag == TAG_IRC)*/
-		mg_create_icon_item (_("_Detach Tab"), GTK_STOCK_REDO, menu,
-									mg_detach_tab_cb, ch);
+	mg_create_icon_item (_("_Detach Tab"), GTK_STOCK_REDO, menu,
+								mg_detach_tab_cb, ch);
 
 	if (sess && tabmenu_list)
 		menu_create (menu, tabmenu_list, sess->channel, FALSE);
+	menu_add_plugin_items (menu, "\x4$TAB");
 
 	if (event->window)
 		gtk_menu_set_screen (GTK_MENU (menu), gdk_drawable_get_screen (event->window));
 	g_object_ref (menu);
-	gtk_object_sink (GTK_OBJECT (menu));
+	g_object_ref_sink (menu);
+	g_object_unref (menu);
 	g_signal_connect (G_OBJECT (menu), "selection-done",
 							G_CALLBACK (mg_menu_destroy), NULL);
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, event->time);
@@ -2797,6 +2773,9 @@ mg_tabwindow_de_cb (GtkWidget *widget, GdkEvent *event, gpointer user_data)
 	GSList *list;
 	session *sess;
 
+	if ((prefs.gui_tray_flags & 1) && tray_toggle_visibility ())
+		return TRUE;
+
 	/* check for remaining toplevel windows */
 	list = sess_list;
 	while (list)
@@ -3225,7 +3204,7 @@ mg_set_title (GtkWidget *vbox, char *title) /* for non-irc tab/window only */
 void
 fe_server_callback (server *serv)
 {
-	joind (1, serv);
+	joind_close (serv);
 
 	if (serv->gui->chanlist_window)
 		mg_close_gen (NULL, serv->gui->chanlist_window);
