@@ -263,23 +263,43 @@ menu_toggle_item (char *label, GtkWidget *menu, void *callback, void *userdata,
 
 static GtkWidget *
 menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags,
-					  gpointer userdata)
+					  gpointer userdata, char *icon)
 {
-	GtkWidget *item;
+	GtkWidget *img, *item;
+
 	if (!label)
 		item = gtk_menu_item_new ();
 	else
 	{
-		if (flags & XCMENU_MARKUP)
+		if (icon)
 		{
-			item = gtk_menu_item_new_with_label ("");
-			gtk_label_set_markup (GTK_LABEL (GTK_BIN (item)->child), label);
-		} else
-		{
-			if (flags & XCMENU_MNEMONIC)
-				item = gtk_menu_item_new_with_mnemonic (label);
+			/*if (flags & XCMENU_MARKUP)
+				item = gtk_image_menu_item_new_with_markup (label);
+			else*/
+				item = gtk_image_menu_item_new_with_mnemonic (label);
+			img = gtk_image_new_from_file (icon);
+			if (img)
+				gtk_image_menu_item_set_image ((GtkImageMenuItem *)item, img);
 			else
-				item = gtk_menu_item_new_with_label (label);
+			{
+				img = gtk_image_new_from_stock (icon, GTK_ICON_SIZE_MENU);
+				if (img)
+					gtk_image_menu_item_set_image ((GtkImageMenuItem *)item, img);
+			}
+		}
+		else
+		{
+			if (flags & XCMENU_MARKUP)
+			{
+				item = gtk_menu_item_new_with_label ("");
+				gtk_label_set_markup_with_mnemonic (GTK_LABEL (GTK_BIN (item)->child), label);
+			} else
+			{
+				if (flags & XCMENU_MNEMONIC)
+					item = gtk_menu_item_new_with_mnemonic (label);
+				else
+					item = gtk_menu_item_new_with_label (label);
+			}
 		}
 	}
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -289,7 +309,7 @@ menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags,
 								G_CALLBACK (popup_menu_cb), cmd);
 	if (flags & XCMENU_SHADED)
 		gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
-	gtk_widget_show (item);
+	gtk_widget_show_all (item);
 
 	return item;
 }
@@ -436,18 +456,18 @@ menu_create (GtkWidget *menu, GSList *list, char *target, int check_path)
 
 		} else if (!strncasecmp (pop->name, "SEP", 3))
 		{
-			menu_quick_item (0, 0, tempmenu, XCMENU_SHADED, 0);
+			menu_quick_item (0, 0, tempmenu, XCMENU_SHADED, 0, 0);
 
 		} else
 		{
 			if (!check_path || pop->cmd[0] != '!')
 			{
-				menu_quick_item (pop->cmd, pop->name, tempmenu, 0, target);
+				menu_quick_item (pop->cmd, pop->name, tempmenu, 0, target, 0);
 			/* check if the program is in path, if not, leave it out! */
 			} else if (is_in_path (pop->cmd))
 			{
 				childcount++;
-				menu_quick_item (pop->cmd, pop->name, tempmenu, 0, target);
+				menu_quick_item (pop->cmd, pop->name, tempmenu, 0, target, 0);
 			}
 		}
 
@@ -477,7 +497,8 @@ menu_popup (GtkWidget *menu, GdkEventButton *event, gpointer objtounref)
 #endif
 
 	g_object_ref (menu);
-	gtk_object_sink (GTK_OBJECT (menu));
+	g_object_ref_sink (menu);
+	g_object_unref (menu);
 	g_signal_connect (G_OBJECT (menu), "selection-done",
 							G_CALLBACK (menu_destroy), objtounref);
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
@@ -505,8 +526,8 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 	if (num_sel > 1)
 	{
 		snprintf (buf, sizeof (buf), "%d nicks selected.", num_sel);
-		menu_quick_item (0, buf, menu, 0, 0);
-		menu_quick_item (0, 0, menu, XCMENU_SHADED, 0);
+		menu_quick_item (0, buf, menu, 0, 0, 0);
+		menu_quick_item (0, 0, menu, XCMENU_SHADED, 0, 0);
 	} else
 	{
 		user = userlist_find (sess, nick);	/* lasttalk is channel specific */
@@ -528,19 +549,19 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 			{
 				snprintf (buf, sizeof (buf), fmt, _("Real Name:"), _("Unknown"));
 			}
-			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
+			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
 
 			snprintf (buf, sizeof (buf), fmt, _("User:"),
 						user->hostname ? user->hostname : _("Unknown"));
-			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
+			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
 
 			snprintf (buf, sizeof (buf), fmt, _("Country:"),
 						user->hostname ? country(user->hostname) : _("Unknown"));
-			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
+			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
 
 			snprintf (buf, sizeof (buf), fmt, _("Server:"),
 						user->servername ? user->servername : _("Unknown"));
-			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
+			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
 
 			if (user->away)
 			{
@@ -552,7 +573,7 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 					free (msg);
 					snprintf (buf, sizeof (buf), fmt, _("Away Msg:"), real);
 					g_free (real);
-					menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
+					menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
 				}
 			}
 
@@ -568,10 +589,10 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 				snprintf (buf, sizeof (buf), fmt, _("Last Msg:"), _("Unknown"));
 			}
 
-			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0);
+			menu_quick_item (0, buf, submenu, XCMENU_MARKUP, 0, 0);
 
 			menu_quick_endsub ();
-			menu_quick_item (0, 0, menu, XCMENU_SHADED, 0);
+			menu_quick_item (0, 0, menu, XCMENU_SHADED, 0, 0);
 		}
 	}
 
@@ -579,6 +600,7 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 		menu_create (menu, popup_list, NULL, FALSE);
 	else
 		menu_create (menu, popup_list, str_copy, FALSE);
+	menu_add_plugin_items (menu, "\x5$NICK");
 	menu_popup (menu, event, NULL);
 }
 
@@ -739,13 +761,13 @@ menu_urlmenu (GdkEventButton *event, char *url)
 		chop = g_utf8_offset_to_pointer (tmp, 48);
 		chop[0] = chop[1] = chop[2] = '.';
 		chop[3] = 0;
-		menu_quick_item (0, tmp, menu, XCMENU_SHADED, 0);
+		menu_quick_item (0, tmp, menu, XCMENU_SHADED, 0, 0);
 		free (tmp);
 	} else
 	{
-		menu_quick_item (0, str_copy, menu, XCMENU_SHADED, 0);
+		menu_quick_item (0, str_copy, menu, XCMENU_SHADED, 0, 0);
 	}
-	menu_quick_item (0, 0, menu, XCMENU_SHADED, 0);
+	menu_quick_item (0, 0, menu, XCMENU_SHADED, 0, 0);
 
 	/* Two hardcoded entries */
 	if (strncmp (str_copy, "irc://", 6) == 0 ||
@@ -756,6 +778,7 @@ menu_urlmenu (GdkEventButton *event, char *url)
 	menu_quick_item_with_callback (copy_to_clipboard_cb, _("Copy Selected Link"), menu, str_copy);
 	/* custom ones from urlhandlers.conf */
 	menu_create (menu, urlhandler_list, str_copy, TRUE);
+	menu_add_plugin_items (menu, "\x4$URL");
 	menu_popup (menu, event, NULL);
 }
 
@@ -805,8 +828,8 @@ menu_chanmenu (struct session *sess, GdkEventButton * event, char *chan)
 
 	menu = gtk_menu_new ();
 
-	menu_quick_item (0, chan, menu, XCMENU_SHADED, str_copy);
-	menu_quick_item (0, 0, menu, XCMENU_SHADED, str_copy);
+	menu_quick_item (0, chan, menu, XCMENU_SHADED, str_copy, 0);
+	menu_quick_item (0, 0, menu, XCMENU_SHADED, str_copy, 0);
 
 	if (!is_joined)
 		menu_quick_item_with_callback (menu_chan_join, _("Join Channel"), menu,
@@ -846,7 +869,7 @@ static void
 usermenu_create (GtkWidget *menu)
 {
 	menu_create (menu, usermenu_list, "", FALSE);
-	menu_quick_item (0, 0, menu, XCMENU_SHADED, 0);	/* sep */
+	menu_quick_item (0, 0, menu, XCMENU_SHADED, 0, 0);	/* sep */
 	menu_quick_item_with_callback (menu_usermenu, _("Edit This Menu..."), menu, 0);
 }
 
@@ -1072,9 +1095,9 @@ menu_join (GtkWidget * wid, gpointer none)
 }
 
 static void
-menu_away (GtkWidget * wid, gpointer none)
+menu_away (GtkCheckMenuItem *item, gpointer none)
 {
-	handle_command (current_sess, "away", FALSE);
+	handle_command (current_sess, item->active ? "away" : "back", FALSE);
 }
 
 static void
@@ -1524,6 +1547,9 @@ menu_foreach_gui (menu_entry *me, void (*callback) (GtkWidget *, menu_entry *))
 	int tabdone = FALSE;
 	session *sess;
 
+	if (!me->is_main)
+		return;	/* not main menu */
+
 	while (list)
 	{
 		sess = list->data;
@@ -1553,6 +1579,23 @@ menu_update_cb (GtkWidget *menu, menu_entry *me)
 	}
 }
 
+/* radio state changed via mouse click */
+static void
+menu_radio_cb (GtkCheckMenuItem *item, menu_entry *me)
+{
+	me->state = 0;
+	if (item->active)
+	{
+		me->state = 1;
+		if (me->cmd)
+			handle_command (current_sess, me->cmd, FALSE);
+	}
+
+	/* update the state, incase this was changed via right-click. */
+	/* This will update all other windows and menu bars */
+	menu_foreach_gui (me, menu_update_cb);
+}
+
 /* toggle state changed via mouse click */
 static void
 menu_toggle_cb (GtkCheckMenuItem *item, menu_entry *me)
@@ -1572,17 +1615,68 @@ menu_toggle_cb (GtkCheckMenuItem *item, menu_entry *me)
 	menu_foreach_gui (me, menu_update_cb);
 }
 
+static GtkWidget *
+menu_radio_item (char *label, GtkWidget *menu, void *callback, void *userdata,
+						int state, char *groupname)
+{
+	GtkWidget *item;
+	GtkMenuItem *parent;
+	GSList *grouplist = NULL;
+
+	parent = menu_find_item (menu, groupname);
+	if (parent)
+		grouplist = gtk_radio_menu_item_get_group ((GtkRadioMenuItem *)parent);
+
+	item = gtk_radio_menu_item_new_with_label (grouplist, label);
+	gtk_check_menu_item_set_active ((GtkCheckMenuItem*)item, state);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+							G_CALLBACK (callback), userdata);
+	gtk_widget_show (item);
+
+	return item;
+}
+
+static void
+menu_reorder (GtkMenu *menu, GtkWidget *item, int pos)
+{
+	if (pos == 0xffff)	/* outbound.c uses this default */
+		return;
+
+	if (pos < 0)	/* position offset from end/bottom */
+		gtk_menu_reorder_child (menu, item, (g_list_length (GTK_MENU_SHELL (menu)->children) + pos) - 1);
+	else
+		gtk_menu_reorder_child (menu, item, pos);
+}
+
+static GtkWidget *
+menu_add_radio (GtkWidget *menu, menu_entry *me)
+{
+	GtkWidget *item = NULL;
+	char *path = me->path + me->root_offset;
+
+	if (path[0] != 0)
+		menu = menu_find_path (menu, path);
+	if (menu)
+	{
+		item = menu_radio_item (me->label, menu, menu_radio_cb, me, me->state, me->group);
+		menu_reorder (GTK_MENU (menu), item, me->pos);
+	}
+	return item;
+}
+
 static GtkWidget *
 menu_add_toggle (GtkWidget *menu, menu_entry *me)
 {
 	GtkWidget *item = NULL;
+	char *path = me->path + me->root_offset;
 
-	menu = menu_find_path (menu, me->path);
+	if (path[0] != 0)
+		menu = menu_find_path (menu, path);
 	if (menu)
 	{
 		item = menu_toggle_item (me->label, menu, menu_toggle_cb, me, me->state);
-		if (me->pos != -1)
-			gtk_menu_reorder_child (GTK_MENU (menu), item, me->pos);
+		menu_reorder (GTK_MENU (menu), item, me->pos);
 	}
 	return item;
 }
@@ -1591,13 +1685,14 @@ static GtkWidget *
 menu_add_item (GtkWidget *menu, menu_entry *me)
 {
 	GtkWidget *item = NULL;
+	char *path = me->path + me->root_offset;
 
-	menu = menu_find_path (menu, me->path);
+	if (path[0] != 0)
+		menu = menu_find_path (menu, path);
 	if (menu)
 	{
-		item = menu_quick_item (me->cmd, me->label, menu, me->markup ? XCMENU_MARKUP : XCMENU_MNEMONIC, 0);
-		if (me->pos != -1)
-			gtk_menu_reorder_child (GTK_MENU (menu), item, me->pos);
+		item = menu_quick_item (me->cmd, me->label, menu, me->markup ? XCMENU_MARKUP : XCMENU_MNEMONIC, 0, me->icon);
+		menu_reorder (GTK_MENU (menu), item, me->pos);
 	}
 	return item;
 }
@@ -1606,18 +1701,25 @@ static GtkWidget *
 menu_add_sub (GtkWidget *menu, menu_entry *me)
 {
 	GtkWidget *item = NULL;
+	char *path = me->path + me->root_offset;
+	int pos;
 
-	if (me->path[0] != 0)
-		menu = menu_find_path (menu, me->path);
+	if (path[0] != 0)
+		menu = menu_find_path (menu, path);
 	if (menu)
-		menu_quick_sub (me->label, menu, &item, me->markup ? XCMENU_MARKUP : XCMENU_MNEMONIC, me->pos);
+	{
+		pos = me->pos;
+		if (pos < 0)	/* position offset from end/bottom */
+			pos = g_list_length (GTK_MENU_SHELL (menu)->children) + pos;
+		menu_quick_sub (me->label, menu, &item, me->markup ? XCMENU_MARKUP : XCMENU_MNEMONIC, pos);
+	}
 	return item;
 }
 
 static void
 menu_del_cb (GtkWidget *menu, menu_entry *me)
 {
-	GtkWidget *item = menu_find (menu, me->path, me->label);
+	GtkWidget *item = menu_find (menu, me->path + me->root_offset, me->label);
 	if (item)
 		gtk_widget_destroy (item);
 }
@@ -1627,7 +1729,9 @@ menu_add_cb (GtkWidget *menu, menu_entry *me)
 {
 	GtkWidget *item;
 
-	if (me->ucmd)	/* have unselect-cmd? Must be a toggle item */
+	if (me->group)	/* have a group name? Must be a radio item */
+		item = menu_add_radio (menu, me);
+	else if (me->ucmd)	/* have unselect-cmd? Must be a toggle item */
 		item = menu_add_toggle (menu, me);
 	else if (me->cmd || !me->label)	/* label=NULL for separators */
 		item = menu_add_item (menu, me);
@@ -1676,7 +1780,7 @@ fe_menu_update (menu_entry *me)
 /* used to add custom menus to the right-click menu */
 
 static void
-menu_add_plugin_items (GtkWidget *menu)
+menu_add_plugin_mainmenu_items (GtkWidget *menu)
 {
 	GSList *list;
 	menu_entry *me;
@@ -1685,7 +1789,24 @@ menu_add_plugin_items (GtkWidget *menu)
 	while (list)
 	{
 		me = list->data;
-		menu_add_cb (menu, me);
+		if (me->is_main)
+			menu_add_cb (menu, me);
+		list = list->next;
+	}
+}
+
+void
+menu_add_plugin_items (GtkWidget *menu, char *root)
+{
+	GSList *list;
+	menu_entry *me;
+
+	list = menu_list;	/* outbound.c */
+	while (list)
+	{
+		me = list->data;
+		if (!me->is_main && !strncmp (me->path, root + 1, root[0]))
+			menu_add_cb (menu, me);
 		list = list->next;
 	}
 }
@@ -1895,7 +2016,7 @@ togitem:
 				if (menu)
 				{
 					gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
-					menu_add_plugin_items (menu_bar);
+					menu_add_plugin_mainmenu_items (menu_bar);
 				}
 				if (usermenu)
 					usermenu_create (usermenu);
